{% extends "admin/change_list.html" %}
{% load i18n %}

{% block object-tools-items %}

<li>
  <a href="{% url 'admin:trackstats_graph_by_date' %}">
    {% trans 'Charts' %}
  </a>
</li>

{{ block.super}}
{% endblock %}
